@@ -886,7 +886,7 @@ class TimeSeries(Array):
         fft(tmp, f)
         return f
 
-    def inject(self, other):
+    def inject(self, other, copy=True):
         """Return copy of self with other injected into it.
 
         The other vector will be resized and time shifted with sub-sample
@@ -896,18 +896,22 @@ class TimeSeries(Array):
         # only handle equal sample rate for now.
         if not self.sample_rate_close(other):
             raise ValueError('Sample rate must be the same')
-
+        # determine if we want to inject in place or not
+        if copy:
+            ts = self.copy()
+        else:
+            ts = self
         # Other is disjoint
-        if ((other.start_time >= self.end_time) or
-           (self.start_time > other.end_time)):
-            return self.copy()
+        if ((other.start_time >= ts.end_time) or
+           (ts.start_time > other.end_time)):
+            return ts
 
         other = other.copy()
-        dt = float((other.start_time - self.start_time) * self.sample_rate)
+        dt = float((other.start_time - ts.start_time) * ts.sample_rate)
 
         # This coaligns other to the time stepping of self
         if not dt.is_integer():
-            diff = (dt - _numpy.floor(dt)) * self.delta_t
+            diff = (dt - _numpy.floor(dt)) * ts.delta_t
 
             # insert zeros at end
             other.resize(len(other) + (len(other) + 1) % 2 + 1)
@@ -917,7 +921,7 @@ class TimeSeries(Array):
 
         # get indices of other with respect to self
         # this is already an integer to floating point precission
-        left = float(other.start_time - self.start_time) * self.sample_rate
+        left = float(other.start_time - ts.start_time) * ts.sample_rate
         left = int(round(left))
         right = left + len(other)
 
@@ -930,11 +934,10 @@ class TimeSeries(Array):
             left = 0
 
         # other overhangs on right so truncate
-        if right > len(self):
-            oright = len(other) - (right - len(self))
-            right = len(self)
+        if right > len(ts):
+            oright = len(other) - (right - len(ts))
+            right = len(ts)
 
-        ts = self.copy()
         ts[left:right] += other[oleft:oright]
         return ts
 
