@@ -496,23 +496,68 @@ def setup_datafind_runtime_cache_single_call_perifo(cp, scienceSegs, outputDir,
     logging.info("Querying datafind server for all science segments.")
     for ifo, scienceSegsIfo in scienceSegs.items():
         observatory = ifo[0].upper()
-        frameType = cp.get_opt_tags("workflow-datafind",
-                                "datafind-%s-frame-type" % (ifo.lower()), tags)
-        # This REQUIRES a coalesced segment list to work
-        startTime = int(scienceSegsIfo[0][0])
-        endTime = int(scienceSegsIfo[-1][1])
-        try:
-            cache, cache_file = run_datafind_instance(cp, outputDir, connection,
-                                       observatory, frameType, startTime,
-                                       endTime, ifo, tags=tags)
-        except:
-            connection = setup_datafind_server_connection(cp, tags=tags)
-            cache, cache_file = run_datafind_instance(cp, outputDir, connection,
-                                       observatory, frameType, startTime,
-                                       endTime, ifo, tags=tags)
+        checked_times = segments.segmentlist([])
+        frame_types = cp.get_opt_tags(
+            "workflow-datafind",
+            "datafind-%s-frame-type" % (ifo.lower()), tags
+        )
+        # Check if this is one type, or time varying
+        frame_types = frame_types.replace(' ', '').strip().split(',')
+        for ftype in frame_types:
+            # Check the times, default to full time initially
+            # This REQUIRES a coalesced segment list to work
+            start = int(scienceSegsIfo[0][0])
+            end = int(scienceSegsIfo[-1][1])
+            # Then check for limits. We're expecting something like:
+            # value[start:end], so need to extract value, start and end
+            if '[' in ftype:
+                # This gets start and end out
+                bopt = ftype.split('[')[1].split(']')[0]
+                newstart, newend = bopt.split(':')
+                # Then check if the times are within science time
+                start = max(int(newstart), start)
+                end = min(int(newend), end)
+                if end <= start:
+                    continue
+                # This extracts value
+                ftype = ftype.split('[')[0]
+            curr_times = segments.segment(start, end)
+            # The times here must be distinct. We cannot have two different
+            # frame files at the same time from the same ifo.
+            if checked_times.intersects_segment(curr_times):
+                err_msg = "Different frame types cannot overlap in time."
+                raise ValueError(err_msg)
+            checked_times.append(curr_times)
 
-        datafindouts.append(cache_file)
-        datafindcaches.append(cache)
+            # Ask datafind where the frames are
+            try:
+                cache, cache_file = run_datafind_instance(
+                    cp,
+                    outputDir,
+                    connection,
+                    observatory,
+                    ftype,
+                    start,
+                    end,
+                    ifo,
+                    tags=tags
+                )
+            except:
+                connection = setup_datafind_server_connection(cp, tags=tags)
+                cache, cache_file = run_datafind_instance(
+                    cp,
+                    outputDir,
+                    connection,
+                    observatory,
+                    ftype,
+                    start,
+                    end,
+                    ifo,
+                    tags=tags
+                )
+
+            datafindouts.append(cache_file)
+            datafindcaches.append(cache)
     return datafindcaches, datafindouts
 
 def setup_datafind_runtime_frames_single_call_perifo(cp, scienceSegs,
